==> local-booter-25.1.0-129.113-sources.jar: com/xebialabs/deployit/booter/local/ValidationRuleConverter.java <==
package com.xebialabs.deployit.booter.local;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;

import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.validation.ApplicableTo;
import com.xebialabs.deployit.plugin.api.validation.Rule;
import com.xebialabs.deployit.plugin.api.validation.Validator;
import com.xebialabs.xlplatform.synthetic.ValidationRuleSpecification;

import nl.javadude.scannit.Scannit;

import static com.xebialabs.deployit.booter.local.utils.CheckUtils.checkArgument;
import static com.xebialabs.deployit.booter.local.utils.ReflectionUtils.handleInvocationTargetException;
import static com.xebialabs.deployit.booter.local.utils.ReflectionUtils.setField;
import static com.xebialabs.deployit.booter.local.utils.ReflectionUtils.setFieldWithConversion;

public class ValidationRuleConverter {

    private static final Map<String, Class<?>> ruleTypeMap = new HashMap<>();

    private ValidationRuleConverter() {
    }

    static {
        initialize();
    }

    public static void initialize() {
        ruleTypeMap.clear();
        Set<Class<?>> rules = Scannit.getInstance().getTypesAnnotatedWith(Rule.class);
        for (Class<?> validationRuleAnnotation : rules) {
            Rule rule = validationRuleAnnotation.getAnnotation(Rule.class);
            ruleTypeMap.put(rule.type(), validationRuleAnnotation);
        }
    }

    public static Validator<?> makeRule(Annotation validationRuleAnnotation, Type ciType) {
        Class<? extends Annotation> vraClazz = validationRuleAnnotation.annotationType();
        checkApplicability(vraClazz, ciType);
        return createRule(validationRuleAnnotation, vraClazz);
    }

    public static Validator<?> makeRule(ValidationRuleSpecification ruleSpec, Type ciType) {
        Class<?> vraClazz = getValidationRuleAnnotationClass(ruleSpec, ciType.toString());
        checkApplicability(vraClazz, ciType);
        return createRule(ruleSpec, vraClazz);
    }

    public static Validator<?> makeRule(Annotation validationRuleAnnotation, LocalPropertyDescriptor propertyDescriptor) {
        Class<? extends Annotation> vraClazz = validationRuleAnnotation.annotationType();
        checkApplicability(vraClazz, propertyDescriptor);
        return createRule(validationRuleAnnotation, vraClazz);
    }

    public static Validator<?> makeRule(ValidationRuleSpecification ruleSpec, LocalPropertyDescriptor propertyDescriptor) {
        Class<?> vraClazz = getValidationRuleAnnotationClass(ruleSpec, propertyDescriptor.getFqn());
        checkApplicability(vraClazz, propertyDescriptor);
        return createRule(ruleSpec, vraClazz);
    }

    public static boolean isRule(Annotation annotation) {
        return annotation.annotationType().isAnnotationPresent(Rule.class);
    }

    private static Class<?> getValidationRuleAnnotationClass(ValidationRuleSpecification ruleSpec, String fqn) {
        String type = ruleSpec.getType();
        Class<?> vraClazz = ruleTypeMap.get(type);

        if (vraClazz == null) {
            throw new IllegalArgumentException(String.format("Could not find validation rule type %s defined on property %s", type, fqn));
        }
        return vraClazz;
    }

    private static Validator<?> createRule(final Annotation validationRuleAnnotation, final Class<?> vraClazz) {
        Rule rule = vraClazz.getAnnotation(Rule.class);
        Validator<?> validationRule = instantiate(rule.clazz());
        for (Method method : vraClazz.getDeclaredMethods()) {
            Object annotationValue = getAnnotationValue(validationRuleAnnotation, method);
            setField(validationRule, method.getName(), annotationValue);
        }
        return validationRule;
    }

    private static Validator<?> createRule(ValidationRuleSpecification ruleSpec, Class<?> vraClazz) {
        Validator<?> validationRule = instantiate(vraClazz.getAnnotation(Rule.class).clazz());

        ruleSpec.getProperties().forEach((key, value) ->
            setFieldWithConversion(validationRule, key, value));

        return validationRule;
    }

    private static void checkApplicability(final Class<?> vraClazz, final Type ciType) {
        if (vraClazz.isAnnotationPresent(Target.class)) {
            Target annotation = vraClazz.getAnnotation(Target.class);
            checkArgument(Arrays.asList(annotation.value()).contains(ElementType.TYPE), "Validation rule [%s] is not applicable to type [%s], because it is not a type level validation", vraClazz.getAnnotation(Rule.class).type(), ciType);
        }
    }

    private static void checkApplicability(final Class<?> vraClazz, final LocalPropertyDescriptor propertyDescriptor) {
        if (vraClazz.isAnnotationPresent(ApplicableTo.class)) {
            List<PropertyKind> propertyKinds = Arrays.asList(vraClazz.getAnnotation(ApplicableTo.class).value());
            checkArgument(propertyKinds.contains(propertyDescriptor.getKind()), "Validation rule [%s] is not applicable to property [%s], because it is not of type(s) %s", vraClazz.getAnnotation(Rule.class).type(), propertyDescriptor.getFqn(), propertyKinds);
        }

        if (vraClazz.isAnnotationPresent(Target.class)) {
            Target annotation = vraClazz.getAnnotation(Target.class);
            checkArgument(Arrays.asList(annotation.value()).contains(ElementType.FIELD), "Validation rule [%s] is not applicable to property [%s], because it is not a property level validation", vraClazz.getAnnotation(Rule.class).type(), propertyDescriptor.getFqn());
        }
    }

    private static Validator<?> instantiate(Class<? extends Validator<?>> rule) {
        try {
            return rule.getDeclaredConstructor().newInstance();
        } catch (ReflectiveOperationException e) {
            throw new IllegalStateException("Cannot instantiate validation rule " + rule, e);
        }
    }

    private static Object getAnnotationValue(Annotation validationRuleAnnotation, Method method) {
        try {
            return method.invoke(validationRuleAnnotation);
        } catch (IllegalAccessException e) {
            throw new IllegalStateException("Could not invoke method.", e);
        } catch (InvocationTargetException e) {
            throw handleInvocationTargetException(e, "Could not convert validation rule.");
        }
    }
}

==> local-booter-2015.2.6-sources.jar: com/xebialabs/deployit/booter/local/ValidationRuleConverter.java <==
package com.xebialabs.deployit.booter.local;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Map;
import java.util.Set;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import com.google.common.collect.Maps;

import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.validation.ApplicableTo;
import com.xebialabs.deployit.plugin.api.validation.Rule;
import com.xebialabs.deployit.plugin.api.validation.Validator;

import nl.javadude.scannit.Scannit;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.collect.Lists.newArrayList;
import static com.xebialabs.deployit.booter.local.utils.ReflectionUtils.*;

class ValidationRuleConverter {

    public static final String RULE_ELEMENT_NAME = "rule";

    private static final Map<String, Class<?>> ruleTypeMap = Maps.newHashMap();

    static {
        Set<Class<?>> rules = Scannit.getInstance().getTypesAnnotatedWith(Rule.class);
        for (Class<?> validationRuleAnnotation : rules) {
            Rule rule = validationRuleAnnotation.getAnnotation(Rule.class);
            ruleTypeMap.put(rule.type(), validationRuleAnnotation);
        }
    }

    static Validator<?> makeRule(Annotation validationRuleAnnotation, Type ciType) {
        Class<? extends Annotation> vraClazz = validationRuleAnnotation.annotationType();
        checkApplicability(vraClazz, ciType);
        return createRule(validationRuleAnnotation, vraClazz);
    }

    static Validator<?> makeRule(Element element, Type ciType) {
        Class<?> vraClazz = getValidationRuleAnnotationClass(element, ciType.toString());
        checkApplicability(vraClazz, ciType);
        return createRule(element, vraClazz);
    }

    static Validator<?> makeRule(Annotation validationRuleAnnotation, LocalPropertyDescriptor propertyDescriptor) {
        Class<? extends Annotation> vraClazz = validationRuleAnnotation.annotationType();
        checkApplicability(vraClazz, propertyDescriptor);
        return createRule(validationRuleAnnotation, vraClazz);
    }

    static Validator<?> makeRule(Element element, LocalPropertyDescriptor propertyDescriptor) {
        Class<?> vraClazz = getValidationRuleAnnotationClass(element, propertyDescriptor.getFqn());
        checkApplicability(vraClazz, propertyDescriptor);
        return createRule(element, vraClazz);
    }

    static boolean isRule(Annotation annotation) {
        return annotation.annotationType().isAnnotationPresent(Rule.class);
    }

    private static Class<?> getValidationRuleAnnotationClass(final Element element, final String fqn) {
        String type = element.getAttribute("type");
        Class<?> vraClazz = ruleTypeMap.get(type);

        if (vraClazz == null) {
            throw new IllegalArgumentException(String.format("Could not find validation rule type %s defined on property %s", type, fqn));
        }
        return vraClazz;
    }

    private static Validator<?> createRule(final Annotation validationRuleAnnotation, final Class<?> vraClazz) {
        Rule rule = vraClazz.getAnnotation(Rule.class);
        Validator<?> validationRule = instantiate(rule.clazz());
        for (Method method : vraClazz.getDeclaredMethods()) {
            Object annotationValue = getAnnotationValue(validationRuleAnnotation, method);
            setField(validationRule, method.getName(), annotationValue);
        }
        return validationRule;
    }

    private static Validator<?> createRule(Element element, Class<?> vraClazz) {
        Validator<?> validationRule = instantiate(vraClazz.getAnnotation(Rule.class).clazz());
        NamedNodeMap attributes = element.getAttributes();
        for (int i = 0; i < attributes.getLength(); i++) {
            Attr attr = (Attr) attributes.item(i);
            String name = attr.getName();
            if (!name.equals("type")) {
                setFieldWithConversion(validationRule, name, attr.getValue());
            }
        }
        return validationRule;
    }

    private static void checkApplicability(final Class<?> vraClazz, final Type ciType) {
        if (vraClazz.isAnnotationPresent(Target.class)) {
            Target annotation = vraClazz.getAnnotation(Target.class);
            checkArgument(newArrayList(annotation.value()).contains(ElementType.TYPE), "Validation rule [%s] is not applicable to type [%s], because it is not a type level validation", vraClazz.getAnnotation(Rule.class).type(), ciType);
        }
    }

    private static void checkApplicability(final Class<?> vraClazz, final LocalPropertyDescriptor propertyDescriptor) {
        if (vraClazz.isAnnotationPresent(ApplicableTo.class)) {
            ArrayList<PropertyKind> propertyKinds = newArrayList(vraClazz.getAnnotation(ApplicableTo.class).value());
            checkArgument(propertyKinds.contains(propertyDescriptor.getKind()), "Validation rule [%s] is not applicable to property [%s], because it is not of type(s) %s", vraClazz.getAnnotation(Rule.class).type(), propertyDescriptor.getFqn(), propertyKinds);
        }

        if (vraClazz.isAnnotationPresent(Target.class)) {
            Target annotation = vraClazz.getAnnotation(Target.class);
            checkArgument(newArrayList(annotation.value()).contains(ElementType.FIELD), "Validation rule [%s] is not applicable to property [%s], because it is not a property level validation", vraClazz.getAnnotation(Rule.class).type(), propertyDescriptor.getFqn());
        }
    }

    private static Validator<?> instantiate(Class<? extends Validator<?>> rule) {
        try {
            return rule.newInstance();
        } catch (InstantiationException e) {
            throw new IllegalStateException("Cannot instantiate validation rule " + rule, e);
        } catch (IllegalAccessException e) {
            throw new IllegalStateException("Cannot instantiate validation rule " + rule, e);
        }
    }

    private static Object getAnnotationValue(Annotation validationRuleAnnotation, Method method) {
        try {
            return method.invoke(validationRuleAnnotation);
        } catch (IllegalAccessException e) {
            throw new IllegalStateException("Could not invoke method.", e);
        } catch (InvocationTargetException e) {
            throw handleInvocationTargetException(e, "Could not convert validation rule.");
        }
    }
}
